import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';
import { generalScroller, generalPlaceholder, generalMatching, generalTooltip, generalGutter, generalLine, generalPanel, generalSearchField, generalCursor, generalContent } from '@fsegurai/codemirror-theme-utils';

/**
 * Enhanced Material Dark theme color definitions
 * --------------------------------------------
 * Colors organized by function with visual color blocks
 */
// Base colors
const base00 = '#212121'; // Background
const base01 = '#505d64'; // Lighter background (popups, statuslines)
const base02 = '#606f7a'; // Selection background
const base03 = '#707d8b'; // Comments, invisibles, line highlighting
const base04 = '#a0a4ae'; // Dark foreground (cursor)
const base05 = '#bdbdbd'; // Default foreground
const base06 = '#e0e0e0'; // Light foreground
const base07 = '#202325'; // Dark background (gutter)
// Accent colors
const base_red = '#ff5f52'; // Keywords, storage, errors
const base_deeporange = '#ff6e40'; // Constants, attributes
const base_pink = '#fa5788'; // Regex, special symbols
const base_yellow = '#facf4e'; // Classes, numbers
const base_orange = '#ffad42'; // Strings, values
const base_cyan = '#56c8d8'; // Support, functions
const base_indigo = '#7186f0'; // Variables, parameters
const base_purple = '#cf6edf'; // Operators, tags
const base_green = '#6abf69'; // Added elements
const base_lightgreen = '#99d066'; // Modified elements
const base_teal = '#4ebaaa'; // Markup headings
// UI specific colors
const invalid = base_red;
const darkBackground = base07;
const highlightBackground = '#2d333b30'; // Line highlight with transparency
const background = base00;
const tooltipBackground = base01;
const selection = '#ffffff1f'; // Selection background with transparency
const selectionMatch = '#4A707A80'; // Selection match background with transparency
const cursor = base04; // Cursor color
const activeBracketBg = '#39496650'; // Active bracket background with transparency
const activeBracketBorder = base_cyan; // Active bracket border
const diagnosticWarning = base_orange; // Warning color
const linkColor = base_cyan; // Link color
const visitedLinkColor = base_purple; // Visited link color
/**
 * Enhanced editor theme styles for Material Dark
 */
const materialDarkTheme = /*@__PURE__*/EditorView.theme({
    // Base editor styles
    '&': {
        color: base05,
        backgroundColor: background,
        fontSize: generalContent.fontSize,
        fontFamily: generalContent.fontFamily,
    },
    // Content and cursor
    '.cm-content': {
        caretColor: cursor,
        lineHeight: generalContent.lineHeight,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
        borderLeftWidth: generalCursor.borderLeftWidth,
    },
    '.cm-fat-cursor': {
        backgroundColor: `${cursor}99`,
        color: background,
    },
    // Selection
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    // Make sure selection appears above active line
    '.cm-selectionLayer': {
        zIndex: 100,
    },
    // Search functionality
    '.cm-searchMatch': {
        backgroundColor: '#394966cc',
        outline: `1px solid ${base_cyan}`,
        color: base06,
        borderRadius: generalSearchField.borderRadius,
        '& span': {
            color: base06,
        },
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base_cyan,
        color: background,
        padding: generalSearchField.padding,
        '& span': {
            color: background,
        },
    },
    '.cm-search.cm-panel.cm-textfield': {
        color: base05,
        borderRadius: generalSearchField.borderRadius,
        padding: generalSearchField.padding,
    },
    // Panels
    '.cm-panels': {
        backgroundColor: base00,
        color: base05,
        borderRadius: '4px',
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: `1px solid ${base03}`,
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: `1px solid ${base03}`,
    },
    '.cm-panel button': {
        backgroundColor: tooltipBackground,
        color: base05,
        border: generalPanel.border,
        borderRadius: generalPanel.borderRadius,
        padding: generalPanel.padding,
    },
    '.cm-panel button:hover': {
        backgroundColor: base02,
    },
    // Line highlighting
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
        borderRadius: generalLine.borderRadius,
        zIndex: 1,
    },
    // Gutters
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03,
        border: generalGutter.border,
        borderRight: `1px solid ${base01}`,
        paddingRight: generalGutter.paddingRight,
    },
    '.cm-activeLineGutter': {
        backgroundColor: '#252a2c',
        color: base05,
        fontWeight: generalGutter.fontWeight,
    },
    '.cm-lineNumbers': {
        fontSize: generalGutter.fontSize,
    },
    '.cm-foldGutter': {
        fontSize: generalGutter.fontSize,
    },
    '.cm-foldGutter .cm-gutterElement': {
        color: base03,
        cursor: 'pointer',
    },
    '.cm-foldGutter .cm-gutterElement:hover': {
        color: base05,
    },
    // Tooltips and autocomplete
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
        border: `1px solid ${base03}`,
        borderRadius: generalTooltip.borderRadius,
        padding: generalTooltip.padding,
        boxShadow: '0 2px 8px rgba(0, 0, 0, 0.3)',
    },
    '.cm-tooltip-autocomplete': {
        '& > ul': {
            backgroundColor: tooltipBackground,
            border: 'none',
        },
        '& > ul > li': {
            padding: generalTooltip.padding,
            lineHeight: generalTooltip.lineHeight,
        },
        '& > ul > li[aria-selected]': {
            backgroundColor: selection,
            color: base06,
            borderRadius: generalTooltip.borderRadiusSelected,
        },
        '& > ul > li > span.cm-completionIcon': {
            color: base03,
            paddingRight: generalTooltip.paddingRight,
        },
        '& > ul > li > span.cm-completionDetail': {
            color: base03,
            fontStyle: 'italic',
        },
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    // Diagnostics styling
    '.cm-diagnostic': {
        '&-error': {
            borderLeft: `3px solid ${invalid}`,
        },
        '&-warning': {
            borderLeft: `3px solid ${diagnosticWarning}`,
        },
        '&-info': {
            borderLeft: `3px solid ${linkColor}`,
        },
    },
    '.cm-lintPoint-error': {
        borderBottom: `2px wavy ${invalid}`,
    },
    '.cm-lintPoint-warning': {
        borderBottom: `2px wavy ${diagnosticWarning}`,
    },
    // Matching brackets
    '.cm-matchingBracket': {
        backgroundColor: activeBracketBg,
        outline: `1px solid ${activeBracketBorder}`,
        borderRadius: generalMatching.borderRadius,
    },
    '.cm-nonmatchingBracket': {
        backgroundColor: '#ff5f5240',
        outline: `1px solid ${invalid}`,
        borderRadius: generalMatching.borderRadius,
    },
    // Selection matches
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
        outline: `1px solid ${base02}50`,
        borderRadius: generalMatching.borderRadius,
    },
    // Fold placeholder
    '.cm-foldPlaceholder': {
        backgroundColor: tooltipBackground,
        color: base03,
        border: `1px dotted ${base03}70`,
        borderRadius: generalPlaceholder.borderRadius,
        padding: generalPlaceholder.padding,
        margin: generalPlaceholder.margin,
    },
    // Focus outline
    '&.cm-focused': {
        outline: 'none',
        boxShadow: `0 0 0 2px ${background}, 0 0 0 3px ${base_cyan}40`,
    },
    // Scrollbars
    '& .cm-scroller::-webkit-scrollbar': {
        width: generalScroller.width,
        height: generalScroller.height,
    },
    '& .cm-scroller::-webkit-scrollbar-track': {
        background: darkBackground,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb': {
        backgroundColor: base02,
        borderRadius: generalScroller.borderRadius,
        border: `3px solid ${darkBackground}`,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb:hover': {
        backgroundColor: base03,
    },
    // Ghost text
    '.cm-ghostText': {
        opacity: '0.5',
        color: base03,
    },
}, { dark: true });
/**
 * Enhanced syntax highlighting for Material Dark theme
 */
const materialDarkHighlightStyle = /*@__PURE__*/HighlightStyle.define([
    // Keywords and control flow
    { tag: tags.keyword, color: base_red, fontWeight: 'bold' },
    { tag: tags.controlKeyword, color: base_red, fontWeight: 'bold' },
    { tag: tags.moduleKeyword, color: base_red, fontWeight: 'bold' },
    // Names and variables
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: base_indigo },
    { tag: [tags.variableName], color: base_lightgreen },
    { tag: [tags.propertyName], color: base_purple, fontStyle: 'normal' },
    // Classes and types
    { tag: [tags.typeName], color: base_red },
    { tag: [tags.className], color: base_yellow, fontStyle: 'italic' },
    { tag: [tags.namespace], color: base_indigo, fontStyle: 'italic' },
    // Operators and punctuation
    { tag: [tags.operator, tags.operatorKeyword], color: base05 },
    { tag: [tags.bracket], color: base03 },
    { tag: [tags.brace], color: base03 },
    { tag: [tags.punctuation], color: base03 },
    // Functions and parameters
    { tag: [/*@__PURE__*/tags.function(tags.variableName)], color: base_cyan },
    { tag: [tags.labelName], color: base_cyan, fontStyle: 'italic' },
    { tag: [/*@__PURE__*/tags.definition(/*@__PURE__*/tags.function(tags.variableName))], color: base_cyan },
    { tag: [/*@__PURE__*/tags.definition(tags.variableName)], color: base_indigo },
    // Constants and literals
    { tag: tags.number, color: base_yellow },
    { tag: tags.changed, color: base_yellow },
    { tag: tags.annotation, color: invalid, fontStyle: 'italic' },
    { tag: tags.modifier, color: base_deeporange, fontStyle: 'italic' },
    { tag: tags.self, color: base_deeporange },
    {
        tag: [tags.color, /*@__PURE__*/tags.constant(tags.name), /*@__PURE__*/tags.standard(tags.name)],
        color: base_deeporange,
    },
    { tag: [tags.atom, tags.bool, /*@__PURE__*/tags.special(tags.variableName)], color: base_deeporange },
    // Strings and regex
    { tag: [tags.processingInstruction, tags.inserted], color: base_green },
    { tag: [/*@__PURE__*/tags.special(tags.string), tags.regexp], color: base_pink },
    { tag: tags.string, color: base_orange },
    // Punctuation and structure
    { tag: /*@__PURE__*/tags.definition(tags.typeName), color: base_red, fontWeight: 'bold' },
    // Comments and documentation
    { tag: tags.meta, color: base03 },
    { tag: tags.comment, fontStyle: 'italic', color: base03 },
    { tag: tags.docComment, fontStyle: 'italic', color: base03 },
    // HTML/XML elements
    { tag: [tags.tagName], color: base_purple },
    { tag: [tags.attributeName], color: base_yellow },
    // Markdown and text formatting
    { tag: [tags.heading], fontWeight: 'bold', color: base_teal },
    { tag: tags.heading1, color: base_yellow },
    { tag: tags.heading2, color: base_orange },
    { tag: tags.heading3, color: base_cyan },
    { tag: tags.heading4, color: base_indigo },
    { tag: tags.heading5, color: base_purple },
    { tag: tags.heading6, color: base_red },
    { tag: [tags.strong], fontWeight: 'bold', color: base_indigo },
    { tag: [tags.emphasis], fontStyle: 'italic', color: base_orange },
    // Links and URLs
    {
        tag: [tags.link],
        color: visitedLinkColor,
        fontWeight: '500',
        textDecoration: 'underline',
        textUnderlinePosition: 'under',
    },
    {
        tag: [tags.url],
        color: linkColor,
        textDecoration: 'underline',
        textUnderlineOffset: '2px',
    },
    // Special states
    {
        tag: [tags.invalid],
        color: base05,
        textDecoration: 'underline wavy',
        borderBottom: `1px wavy ${invalid}`,
    },
    { tag: [tags.strikethrough], color: invalid, textDecoration: 'line-through' },
    // Enhanced syntax highlighting
    { tag: /*@__PURE__*/tags.constant(tags.name), color: base_deeporange },
    { tag: tags.deleted, color: invalid },
    { tag: tags.squareBracket, color: base03 },
    { tag: tags.angleBracket, color: base03 },
    // Additional specific styles
    { tag: tags.monospace, color: base05 },
    { tag: [tags.contentSeparator], color: base_indigo },
    { tag: tags.quote, color: base03 },
]);
/**
 * Combined Material Dark theme extension
 */
const materialDark = [
    materialDarkTheme,
    /*@__PURE__*/syntaxHighlighting(materialDarkHighlightStyle),
];

export { materialDark, materialDarkHighlightStyle, materialDarkTheme };
